#ifndef __FAKEHEAP__
#define __FAKEHEAP__

#include <vector>
#include <algorithm>

/*
 Der Standard-Vektor wird um die typischen PriorityQueue-Funktionen erweitert.
 Es handelt sich aber nicht wirklich um eine PriorityQueue; die Schnittstellen
 dienen lediglich dazu, in bestimmten Programmstellen langsamere echte
 Priorityqueues durch dieses Fake zu ersetzen, sofern bestimmte
 Nebenbedingungen erfllt werden:

 Zugriff ist nur blockweise gestattet.
 Zwischen push-Operationen einerseits und top/pop-Operationen andererseits
 ist "sort" aufzurufen.
*/

#warning "using FakeHeap (experimental feature)"

//#define FAKEHEAP_DEBUG

template <class T> class FakeHeap : public vector<T>
{
private:

#ifdef FAKEHEAP_DEBUG
 mutable bool push_locked;
#endif

public:

#ifdef FAKEHEAP_DEBUG
 inline FakeHeap() : vector<T>(), push_locked(false) { cout << "HELLO!" << endl; };
 inline ~FakeHeap() { };
#endif

#ifdef FAKEHEAP_DEBUG
 inline void clear()
  {
    cout << "FakeHeap:: CLEAR..." << endl;
    vector<T>::clear();
    push_locked=false;
  }
#endif

 inline void sort(void)
  {
//    std::cout << "FakeHeap: sorting " << size() << " elements... " << std::flush;
    std::sort(this->begin(),this->end());
//    std::cout << "done." << std::endl;
#ifdef FAKEHEAP_DEBUG
    push_locked=true;
#endif
  }
 inline void push(const T &x)
  {
#ifdef FAKEHEAP_DEBUG
    if (push_locked)
     { 
       std::cerr << "FakeHeap:: Hey?!!! push is locked!!!" << endl;
       push_back(x);
       sort();
       return;
     }
#endif
    push_back(x);
  }
 inline void pop(void)
  {
#ifdef FAKEHEAP_DEBUG
    if (!push_locked)
     {
       std::cerr << "FakeHeap:: Hey?!!! pop is locked!!!" << endl;
       //exit(1);
     }
    push_locked=true;
#endif
    this->pop_back();
  }
 inline const T& top(void) const
  {
#ifdef FAKEHEAP_DEBUG
    if (!push_locked)
     {
       std::cerr << "FakeHeap:: Hey?!!! top is locked!!!" << endl;
       //exit(1);
     }
    push_locked=true;
#endif
    return this->back();
  } 
};

#endif
